/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

/****************************************************************************
**
** Author: Wolfgang Glas / EV-i
**
** A frontend header for encapsulating fixed-size integral types.
**
*****************************************************************************/

#ifndef FANCYBASE_INT_H
#define FANCYBASE_INT_H

#ifndef __STDC_CONSTANT_MACROS
#define __STDC_CONSTANT_MACROS
#endif
#ifndef __STDC_LIMIT_MACROS
#define __STDC_LIMIT_MACROS
#endif
#include <stdint.h>

// FANCY_INT64_T_FMT is the format specifier for printf et al. for int64_t
// FANCY_UINT64_T_FMT is the format specifier for printf et al. for uint64_t
#ifdef WIN32
#define FANCY_INT64_T_FMT_LENGTH "I64"
#define FANCY_INT64_T_FMT  "%I64d"
#define FANCY_INT64_T_XFMT "%I64X"
#define FANCY_UINT64_T_FMT "%I64u"
#define FANCY_UINT64_T_XFMT "%I64X"
#elif __WORDSIZE == 64
#define FANCY_INT64_T_FMT_LENGTH "l"
#define FANCY_INT64_T_FMT  "%ld"
#define FANCY_INT64_T_XFMT "%lX"
#define FANCY_UINT64_T_FMT "%lu"
#define FANCY_UINT64_T_XFMT "%lX"
#else
#define FANCY_INT64_T_FMT_LENGTH "ll"
#define FANCY_INT64_T_FMT  "%lld"
#define FANCY_INT64_T_XFMT "%llX"
#define FANCY_UINT64_T_FMT "%llu"
#define FANCY_UINT64_T_XFMT "%llX"
#endif

// FANCY_SIZE_T_FMT is the format specifier for printf et al. for size_t
// FANCY_SIZE_T_FMT_ARG casts size_t args for printf et al. in order to avoid gcc warnings.
#ifdef WIN64
#define FANCY_SIZE_T_FMT  "%I64u"
#define FANCY_SIZE_T_XFMT "%I64X"
#define FANCY_SIZE_T_FMT_ARG(a) ((uint64_t)(a))
#define FANCY_SSIZE_T_FMT  "%I64d"
#define FANCY_SSIZE_T_XFMT "%I64X"
#define FANCY_SSIZE_T_FMT_ARG(a) ((int64_t)(a))
#elif defined ( WIN32 )
#define FANCY_SIZE_T_FMT "%lu"
#define FANCY_SIZE_T_XFMT "%lX"
#define FANCY_SIZE_T_FMT_ARG(a) ((unsigned long)(a))
#define FANCY_SSIZE_T_FMT "%ld"
#define FANCY_SSIZE_T_XFMT "%lX"
#define FANCY_SSIZE_T_FMT_ARG(a) ((long)(a))
#else
#define FANCY_SIZE_T_FMT "%zu"
#define FANCY_SIZE_T_XFMT "%zX"
#define FANCY_SIZE_T_FMT_ARG(a) (a)
#define FANCY_SSIZE_T_FMT "%zd"
#define FANCY_SSIZE_T_XFMT "%zX"
#define FANCY_SSIZE_T_FMT_ARG(a) (a)
#endif

#endif
