/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2014 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYJSONOPTIONS_H_
#define FANCYJSONOPTIONS_H_

#include <fancyobject.h>

namespace Fancy {

class String;

class FANCYBASE_API JsonOptions : virtual public Object {

	ObjPtr<String> terminationSequence;
	ObjPtr<String> serialIDproperty;
	bool skipUnderscoreSerialIDs;
	bool prettyPrint;
	// add new properties here to retain binary compatibility.

protected:
	JsonOptions();
	virtual ~JsonOptions();

public:

	typedef ObjPtr<JsonOptions> ptr_t;

	/**
	 * Return a new instance with the default values set:
	 * - prettyPrint              false
	 * - serialIDproperty         "serialID"
	 * - skpiUnderscoreSerialIDSs true
	 * - terminationSequence      null
	 */
	static ptr_t newInstance();

	/**
	 * Whether pretty-printing is enabled.
	 */
	bool isPrettyPrint() const;
	void setPrettyPrint(bool prettyPrint);

	/**
	 * The property to write the serial ID to. Set to null,
	 * if you do not want to write the serial ID of POJOs
	 * to the JSON stream.
	 *
	 * Default value is "serialID".
	 * If set to null, polymorphic class hierarchies might not
	 * correctly be deserialized.
	 */
	const ObjPtr<String>& getSerialIDproperty() const;
	void setSerialIDproperty(const ObjPtr<String>& serialIDproperty);

	/**
	 * Whether to skip internal serialIDs, which start with an underscore.
	 * The default value is true, because some POJOs like JSON-RPC's internal
	 * are defined with and underscore serialID to state that these POJOs
	 * arise from a JSON standard and not from an underlying JAVA class.
	 */
	bool isSkipUnderscoreSerialIDs() const;
	void setSkipUnderscoreSerialIDs(bool skipUnderscoreSerialIDs);

	/**
	 * A termination sequence to set. If set to null, the serializer
	 * reads until the end of the stream.
	 *
	 * Otherwise, the serializer skips whitespace until it meets the
	 * termination sequence in the stream content.
	 * A well-known termination sequence might be "\n" or "\r\n".
	 */
	const ObjPtr<String>& getTerminationSequence() const;
	void setTerminationSequence(const ObjPtr<String>& terminationSequence);
};

} /* namespace Fancy */

#endif /* FANCYJSONOPTIONS_H_ */
