/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYSESSIONCONTEXTFACTORY_H_
#define FANCYSESSIONCONTEXTFACTORY_H_

#include <fancyobject.h>

namespace Fancy {

/**
 * A session context factory is used to instantiate a thread-bound context from
 * a possibly pooled resource like database connections or remoting clients.
 */
class FANCYBASE_API SessionContextFactory: public virtual Fancy::Object {
public:

    typedef ObjPtr<SessionContextFactory> ptr_t;

    /**
     * @return The key which is used to bind a session context to the current thread
     *         using @c Fancy::Thread::setSpecific() inside @c newSessionContext().
     */
    virtual Fancy::Object::ptr_t getSpecificKey() const = 0;

    /**
     * @return A new session context. The context is usually bound to the current thread
     *         using the key returned by @c getSpecificKey(). Pooled resources are pulled from
     *         the appropriate pool.
     *         If the returned object is destroyed, the session context is closed,
     *         the local context is unbound from the current thread and pooled
     *         resources are returned to the client pool.
     */
    virtual Fancy::Object::ptr_t newSessionContext() = 0;

protected:
    SessionContextFactory();
    virtual ~SessionContextFactory();
};

}

#endif /* FANCYSESSIONCONTEXTFACTORY_H_ */
