/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYSET_H_
#define FANCYSET_H_

#include <fancycollection.h>
#include <fancyserializable.h>

namespace Fancy {

class Iterator;
class List;

/*!
 * An interface for an untyped set of objects.
 * This is pretty much like \c java.util.Set and is used in favor
 * to a template in order to avoid code-bloat and to allow a generic
 * deserializer to be implemented.
 */
class FANCYBASE_API Set: public virtual Collection {
public:
    typedef ObjPtr<Set> ptr_t;

    /**
     * Return the serial ID of the declared component type of this list or
     * @c NULL, if the component type is @c Object.
     */
    virtual const char *getComponentSerialID() const = 0;
    /**
     * Return the serialVersionUID of the declared component type of this list or
     * @c (int64_t)-1, if the component type is @c Object.
     */
    virtual int64_t getComponentSerialVersionUID() const = 0;

    virtual ObjPtr_less_t getLess() const = 0;

    virtual size_t getSize() const = 0;

    virtual bool contains(const ObjPtr<Object> &o) const = 0;

    virtual bool remove(const ObjPtr<Object> &o) = 0;

    virtual void add(const ObjPtr<Object> &o) = 0;

    virtual void addString(const std::string &s);
    virtual bool containsString(const std::string &s) const = 0;
    virtual bool removeString(const std::string &s) = 0;
    virtual bool containsString(const char *s) const = 0;
    virtual bool removeString(const char *s) = 0;

    bool lessThan(const Object &o) const;
    bool equals(const Object &o) const;

    virtual ObjPtr<List> getMembers() const = 0;

    virtual ObjPtr<Iterator> getBeginIterator() = 0;
    virtual ObjPtr<Iterator> getEndIterator() = 0;
    virtual ObjPtr<Iterator> getLowerBound(const ObjPtr<Object> &o) = 0;

    static ObjPtr<Set> newInstance();
    static ObjPtr<Set> newInstance(ObjPtr_less_t less);
    static ObjPtr<Set> newInstance(getSerialID_s_t getSerialID_s, getSerialVersionUID_s_t getSerialVersionUID_s);
    static ObjPtr<Set> newInstance(getSerialID_s_t getSerialID_s, getSerialVersionUID_s_t getSerialVersionUID_s, ObjPtr_less_t less);

    template <class T> static ObjPtr<Set> newTypedInstance() {
        return Set::newInstance(T::getSerialID_s,T::getSerialVersionUID_s);
    }

    template <class T> static ObjPtr<Set> newTypedInstance(ObjPtr_less_t less) {
        return Set::newInstance(T::getSerialID_s,T::getSerialVersionUID_s,less);
    }

protected:
    Set();
    virtual ~Set();
};

}

#endif /* FANCYSET_H_ */
