/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

/****************************************************************************
**
** Author: Wolfgang Glas / EV-i
**
** Global signal handling stuff.
** 
*****************************************************************************/

#ifndef FANCYBASE_SIGNAL_H
#define FANCYBASE_SIGNAL_H

#include <sys/types.h>
#include <fancybase.h>

#define FANCY_SIGNAL_HUP   1
#define FANCY_SIGNAL_INT   2
#define FANCY_SIGNAL_QUIT  3
#define FANCY_SIGNAL_ILL   4
#define FANCY_SIGNAL_TRAP  5
#define FANCY_SIGNAL_ABRT  6
#define FANCY_SIGNAL_FPT   8
#define FANCY_SIGNAL_KILL  9
#define FANCY_SIGNAL_SEGV 11
#define FANCY_SIGNAL_PIPE 13
#define FANCY_SIGNAL_ALRM 14
#define FANCY_SIGNAL_TERM 15

namespace Fancy {

  FANCYBASE_API void installSignalHandlers();
  FANCYBASE_API void installNewHandler(size_t reserve);

  FANCYBASE_API void mallocFailHandler();

  /**
   * Enable core dumps by setting the soft limit for
   * core files to unlimited under UNIX. Under windows,
   * log a warning.
   *
   * If setting the soft limit fails, a Fancy::CException
   * is thrown containing the exact error message.
   */
  FANCYBASE_API void enableCoreDumps();

  FANCYBASE_API void sendSignal(int sig);

#ifdef WIN32
  FANCYBASE_API void *getSignalWinEventHandle();
#endif

  FANCYBASE_API void resetHupSignalCatched();
  FANCYBASE_API int getHupSignalCatched();

  FANCYBASE_API void resetStopSignalCatched();
  FANCYBASE_API int getStopSignalCatched();

  FANCYBASE_API void resetUsrSignalCatched();
  FANCYBASE_API int getUsrSignalCatched();

}

#endif // ! FANCYBASE_SIGNAL_H
